cuda的3DArray和TextureObject

  • cudaArray_t:数据存储的对象,用于存储 2D 或 3D 数据;不涉及纹理的采样(即读取数据时的插值、过滤等操作)。

    复制代码
       cudaExtent ca_extent;//定义三维内存区域的结构体
       ca_extent.width = vol_in->dim[0];
       ca_extent.height = vol_in->dim[1];
       ca_extent.depth = vol_in->dim[2];
       cudaChannelFormatDesc ca_descriptor = cudaCreateChannelDesc<float>();//定义cudaArray的数据格式,shape和type
       cudaMalloc3DArray(&state->dev_vol, &ca_descriptor, ca_extent);//分配三维数据块

    cudaMemcpy3DParms cpy_params = { 0 };//三维内存拷贝操作参数的结构体
    cpy_params.extent = ca_extent; //定义三维内存区域的结构体
    cpy_params.kind = cudaMemcpyHostToDevice; // 拷贝类型(主机到设备、设备到设备等)
    cpy_params.dstArray = state->dev_vol;// 目标数组,和 dstPtr 互斥使用

    //http://sites.google.com/site/cudaiap2009/cookbook-1#TOC-CUDA-3D-Texture-Example-Gerald-Dall
    // The pitched pointer is really tricky to get right. We give the
    // pitch of a row, then the number of elements in a row, then the
    // height, and we omit the 3rd dimension.
    cpy_params.srcPtr = make_cudaPitchedPtr((void*)vol_in->data, ca_extent.width * sizeof(float), ca_extent.width, ca_extent.height);// 源指针,和 srcArray 互斥使用

    cudaMemcpy3D(&cpy_params);//把vol数据拷贝到显存

  • cudaTextureObject_t:纹理对象;包含数据的地址信息,还包括了如何访问这些数据的元数据(如过滤模式、边界处理、坐标转换等)。高效的图像处理支持自动插值,只读操作

    // Specify texture
    struct cudaResourceDesc resDesc;//描述资源(如数组、纹理、表面等)的结构体
    memset(&resDesc, 0, sizeof(resDesc));
    resDesc.resType = cudaResourceTypeArray;//资源是一个 3D 数组。
    resDesc.res.array.array = state->dev_vol;// 3D 数组资源

    // Specify texture object parameters
    struct cudaTextureDesc texDesc;//描述纹理(texture)对象属性的结构体
    memset(&texDesc, 0, sizeof(texDesc));
    texDesc.addressMode[0] = cudaAddressModeWrap;//当坐标超出边界时,纹理坐标会循环回到起始位置。
    texDesc.addressMode[1] = cudaAddressModeWrap;//当坐标超出边界时,纹理坐标会循环回到起始位置。
    texDesc.filterMode = cudaFilterModeLinear;//当纹理坐标落在两个像素之间时使用最近邻插值。
    texDesc.readMode = cudaReadModeElementType;//按纹理的元素类型读取
    texDesc.normalizedCoords = 0;//纹理坐标范围通常是像素索引

    state->tex_vol = 0;//只读纹理内存
    cudaCreateTextureObject(&state->tex_vol, &resDesc, &texDesc, NULL);

  • cudaSurfaceObject_t。适合需要频繁修改数据,适合需要频繁修改数据,读写操作

    // Specify texture
    struct cudaResourceDesc resDesc;//描述资源(如数组、纹理、表面等)的结构体
    memset(&resDesc, 0, sizeof(resDesc));
    resDesc.resType = cudaResourceTypeArray;//资源是一个 3D 数组。
    resDesc.res.array.array = state->dev_vol;// 3D 数组资源

    cudaCreateSurfaceObject(&surfObj, &resDesc);

    复制代码
      __global__ void cuda_clear_3d_array_kernel(cudaSurfaceObject_t surf, dim3 kdim)
      {
          // calculate surface coordinates
          unsigned int x = blockIdx.x * blockDim.x + threadIdx.x;
          unsigned int y = blockIdx.y * blockDim.y + threadIdx.y;
          unsigned int z = blockIdx.z * blockDim.z + threadIdx.z;
    
          // write to memeory
          if (x < kdim.x && y < kdim.y && z < kdim.z) {
              surf3Dwrite<float>(0.f, surf, x * 4, y, z, cudaBoundaryModeTrap);
          }
      }
相关推荐
我命由我123455 小时前
CesiumJS 笔记 - 获取容器中心点、Cartesian3 clone 方法、修改 Cartesian3 对象的高度
前端·javascript·css·前端框架·html·html5·js
Hopebearer_6 小时前
页面突然只剩 DOM?一次静态资源版本错配排查
前端·部署
东风破_6 小时前
ESLint 是什么?为什么你的项目需要它?
前端·后端·代码规范
圣殿骑士-Khtangc8 小时前
Go字符串高效拼接性能对比与底层原理分析
服务器·前端·golang
BigTopOne9 小时前
【ijkplayer】 硬解码流程
前端
kyriewen9 小时前
DeepSeek Harness开源第一天我就上手了——和Claude Code的差距比想象中大
前端·ai编程·deepseek
捡田螺的小男孩9 小时前
什么是 Skill?手把手带你写一个简单有用的 Skill!
前端·后端·程序员
IT_陈寒9 小时前
Redis集群这个坑,差点让我通宵
前端·人工智能·后端
90后的晨仔10 小时前
uni-app 路由跳转与页面导航 —— 终极详解(对标 iOS / Android / 鸿蒙三端原生)
前端
Heo10 小时前
大厂前端调试不能只会debugger
前端·javascript·面试